
#include <xen/config.h>
#include <xen/init.h>
#include <xen/lib.h>
#include <xen/sched.h>
#include <xen/domain.h>
#include <xen/event.h>
#include <xen/time.h>
#include <xen/perfc.h>
#include <xen/sched-if.h>
#include <xen/softirq.h>
#include <asm/atomic.h>
#include <xen/errno.h>

struct vm_info;
struct hostsched_info {
    struct vm_info *hi_vm_info;
    int hi_host_halted;
};

static int
hostsched_vcpu_init(struct vcpu *v)
{
    struct hostsched_info *hi;

    printk("*** sched_init_vcpu %d:%d\n", v->domain->domain_id, v->vcpu_id);
    hi = xmalloc(struct hostsched_info);
    if (hi == NULL)
        return -1;

    memset(hi, 0, sizeof(*hi));
    v->sched_priv = hi;

    return 0;
}

static void
hostsched_vcpu_destroy(struct vcpu *v)
{
    xfree(v->sched_priv);
}

static void
hostsched_vcpu_wake(struct vcpu *v)
{
    struct hostsched_info *hi = v->sched_priv;
    ASSERT(hi);
    if (test_and_clear_bit(0, &hi->hi_host_halted)) {
        perfc_incr(hostsched_wake_vm);
        kxen_info.ki_wake_vm(hi->hi_vm_info);
    }
}

static void
hostsched_vcpu_yield(struct vcpu *v) 
{
}

static void
hostsched_init(void)
{
}

void
hostsched_setup_vm(struct vcpu *v, struct vm_info *vm_info)
{
    struct hostsched_info *hi = v->sched_priv;
    ASSERT(hi);
    hi->hi_vm_info = vm_info;
}

void
hostsched_halt_vm(struct vcpu *v)
{
    struct hostsched_info *hi = v->sched_priv;
    ASSERT(hi);
    perfc_incr(hostsched_halt_vm);
    set_bit(0, &hi->hi_host_halted);
    local_irq_enable();
    kxen_info.ki_halt_vm(hi->hi_vm_info);
}

void hostsched_vcpu_softirq(struct vcpu *v)
{

    hostsched_vcpu_wake(v);
}

void
hostsched_dump_vcpu(struct vcpu *v)
{
    struct hostsched_info *hi = v->sched_priv;

    if (hi == NULL || hi->hi_vm_info == NULL)
        return;

    printk("    Notifying vcpu (halted %x, runstate %x, pause %lx/%x/%x/%p)\n",
           hi->hi_host_halted, v->runstate.state, v->pause_flags,
		   atomic_read(&v->pause_count), atomic_read(&v->domain->pause_count),
           v->pauser);
    kxen_info.ki_wake_vm(hi->hi_vm_info);
}

struct scheduler sched_host_def = {
    .name           = "Host Scheduler",
    .opt_name       = "host",
    .sched_id       = XEN_SCHEDULER_HOST,

    .init_vcpu      = hostsched_vcpu_init,
    .destroy_vcpu   = hostsched_vcpu_destroy,

    .wake           = hostsched_vcpu_wake,
    .yield          = hostsched_vcpu_yield,

    .init           = hostsched_init,
};

/*
 * Local variables:
 * mode: C
 * c-set-style: "BSD"
 * c-basic-offset: 4
 * tab-width: 4
 * indent-tabs-mode: nil
 * End:
 */
