
#ifndef _KXEN_LINK_H_
#define _KXEN_LINK_H_

#if defined(KXEN_DEFINE_SYMBOLS_PROTO) || defined(KXEN_DEFINE_SYMBOLS_CODE)

#include <kxen_desc.h>

#define KXEN_PROTOTYPES(type)						\
    type struct kxen_info *kxen_info;					\
    type int kxen_sizeof_struct_page_info;				\
    type void (__cdecl *kxen_start_xen)(void);				\
    type long (__cdecl *kxen_run_vm)(struct vm_info *);			\
    type void (__cdecl *kxen_dispatch_ipi)(unsigned int);		\
    type void (__cdecl *kxen_do_softirq)(void);				\
    type void (__cdecl *kxen_handle_keypress)(unsigned char);		\
    type void (__cdecl *kxen_shutdown_xen)(void);			\
    type long (__cdecl *kxen_do_hypercall)(struct kxen_hypercall_desc *); \
    type long (__cdecl *kxen_do_memop)(struct kxen_memop_desc *);	\
    type long (__cdecl *kxen_do_hvmop)(struct kxen_hvmop_desc *);	\
    type long (__cdecl *kxen_do_domctl)(struct kxen_domctl_desc *);

#endif

#if defined(KXEN_DEFINE_SYMBOLS_CODE)
#include <public/libelf.h>

#ifndef PRIx64
#if defined(_WIN32)
#define PRIx64 "I64x"
#elif defined(__APPLE__) && defined(KERNEL)
#define PRIx64 "qx"
#else
#error unsupported os
#endif
#endif

static int get_elf_sym(struct elf_binary *elf, unsigned char *hv,
		       const char *n, int vsize, void *vp)
{
    uint64_t v;
    uint64_t base;
    int shndx;
    const elf_sym *sym;
    const elf_shdr *shdr;

    sym = elf_sym_by_name(elf, n);
    if (sym == NULL)
	return -1;
    shndx = elf_uval(elf, sym, st_shndx);
    switch (shndx) {
    case SHN_UNDEF:
    case SHN_ABS:
	base = 0;
	break;
    case SHN_COMMON:
	return -1;
	break;
    default:
	shdr = elf_shdr_by_index(elf, shndx);
	if ( shdr == NULL )
	    return -1;
	base = elf_uval(elf, shdr, sh_addr) + (uint64_t)(uintptr_t)hv;
	break;
    }
    v = elf_uval(elf, sym, st_value) + base;
    dprintk("sym %s = %" PRIx64 " (base %" PRIx64 ")\n", n, v, base);
    memcpy(vp, &v, vsize);
    return 0;
}

#define GET_ELF_SYM(n, t, v) do {				\
	int ret = get_elf_sym(elf, hv, n, sizeof(t), &v);	\
	if (ret) {						\
	    if (missing_symbol)					\
		*missing_symbol = n;				\
	    return ret;						\
	}							\
} while (/* CONSTCOND */0)

#define KXEN_GET_SYMS(fn_name) int					\
    fn_name(struct elf_binary *elf, unsigned char *hv,			\
            const char **missing_symbol)				\
{									\
									\
    GET_ELF_SYM("kxen_info", struct kxen_info *, kxen_info);		\
    GET_ELF_SYM("__start_xen", void (__cdecl *)(void), kxen_start_xen); \
    GET_ELF_SYM("kxen_run_vm", long (__cdecl *)(struct vm_info *),	\
                kxen_run_vm);						\
    GET_ELF_SYM("kxen_dispatch_ipi", void (__cdecl *)(unsigned int),	\
                kxen_dispatch_ipi);					\
    GET_ELF_SYM("kxen_do_softirq", void (__cdecl *)(void), kxen_do_softirq); \
    GET_ELF_SYM("__sizeof_struct_page_info", int,			\
                kxen_sizeof_struct_page_info);				\
    GET_ELF_SYM("kxen_handle_keypress", void (__cdecl *)(unsigned char), \
                kxen_handle_keypress);					\
    GET_ELF_SYM("kxen_shutdown_xen",					\
		void (__cdecl *)(void), kxen_shutdown_xen);		\
    GET_ELF_SYM("kxen_do_hypercall",					\
		long (__cdecl *)(struct kxen_hypercall_desc *),		\
		kxen_do_hypercall);					\
    GET_ELF_SYM("kxen_do_memop", long (__cdecl *)(struct kxen_memop_desc *), \
                kxen_do_memop);						\
    GET_ELF_SYM("kxen_do_hvmop", long (__cdecl *)(struct kxen_hvmop_desc *), \
		kxen_do_hvmop);						\
    GET_ELF_SYM("kxen_do_domctl",					\
		long (__cdecl *)(struct kxen_domctl_desc *), kxen_do_domctl); \
    return 0;								\
}
#endif

#endif
