
#ifndef _KXEN_H_
#define _KXEN_H_

#include <stdarg.h>
#include <ntifs.h>
#include <ntddk.h>
#include <xen/types.h>

#include <kxen_info.h>

#define KERNEL 1

#ifdef DBG
#define KXEN_DPRINTK
#endif

#define	KXEN_DRIVER_VERSION_MAJOR	0x00000000
#define	KXEN_DRIVER_VERSION_MINOR	0x00000000
#define	KXEN_DRIVER_VERSION_TAG		"preview"

#define KXEN_DEVICE_NAME_U L"\\Device\\kxen"
#define KXEN_DEVICE_NAME_DOS_U L"\\DosDevices\\kxen"

#define KXEN_POOL_TAG 'kxen'

#define MAX_MACHINE_FRAME 0x100000;

// Windows system clock increments in 100ns units, ie 10,000,000Hz.
#define KXEN_HOST_TIME_FREQUENCY 10000000

#define TIME_RELATIVE(x)         (-((int64_t)(x)))
#define TIME_NANO100_PER_MS      10000
#define TIME_NANO100_PER_NANO(x) ((x) / 100)
#define TIME_MS(x)               ((x) * TIME_NANO100_PER_MS)
#define TIME_NANO(x)             (TIME_NANO100_PER_NANO(x))

typedef struct device_extension {
    KSPIN_LOCK de_biglock;
    FAST_IO_DISPATCH de_FastIoDispatch;
    KEVENT de_runnable;
    int de_nrpages;
    struct vm_info de_vm_info;
    int de_nrmdls;
    MDL **de_mdls;
};

/* kxen.c */
extern struct device_extension *kxen_devext;
extern uint8_t *kxen_hv;
extern size_t kxen_size;

/* kxen_cpu.c */
extern unsigned long kxen_cpu_vm;
extern unsigned long kxen_first_cpu(void);
extern void kxen_cpu_pin(unsigned long);
extern void kxen_cpu_pin_current(void);
extern void kxen_cpu_pin_first(void);
#define kxen_cpu_pin_dom0() kxen_cpu_pin_first()
#define kxen_cpu_pin_vm() kxen_cpu_pin(kxen_cpu_vm)
extern void kxen_cpu_unpin(void);
extern void kxen_set_cpu_active_mask(void *, int);
extern void __cdecl kxen_on_each_cpu(void (*)(void *), void *);

/* kxen_ioctl.c */
extern BOOLEAN kxen_ioctl(__in FILE_OBJECT *, __in BOOLEAN, __in_opt VOID *,
			  __in ULONG, __out_opt VOID *, __in ULONG, __in ULONG,
			  __out IO_STATUS_BLOCK *, __in DEVICE_OBJECT *);

/* kxen_load.c */
int kxen_load(struct kxen_load_desc *);
int kxen_unload(void);

/* kxen_mem.c */
void *kernel_malloc(size_t);
void kernel_free(void *, size_t);
uint32_t *kernel_get_mfn_list(uintptr_t, uint32_t);
int kxen_mmapbatch(struct kxen_mmapbatch_desc *);
int kxen_munmap(struct kxen_munmap_desc *);
int kxen_create_vmappings(struct kxen_vmappings_desc *,
			  struct device_extension *);
void kxen_free_vmappings(struct device_extension *);
void * __cdecl kxen_map_pages(unsigned int, void *);
int __cdecl kxen_unmap_pages(unsigned int, void *);

/* kxen_ops.c */
int kxen_except_handler(unsigned int, struct _EXCEPTION_POINTERS *);
void kxen_init_free_allocs(void);
int kxen_init(void);
int kxen_shutdown(void);
int kxen_version(struct kxen_version_desc *);
int kxen_execute(void);

/* kxen_debug.c */
struct device_extension *kdbgdevext;
int kdbgprint;
extern int __cdecl kxen_dprintk(const char *, ...);
#ifdef KXEN_DPRINTK
#define dprintk(fmt, ...) kxen_dprintk(fmt, __VA_ARGS__)
#else
#define dprintk(fmt, ...) do {} while (0)
#endif
#define fail_msg(fmt, ...) kxen_printk("kxen: " fmt "\n", __VA_ARGS__)
extern int kxen_printk(const char *, ...);
extern void kdbgreboot(void);

#endif
